@@ -13,8 +13,6 @@
     #include <nc3internal.h>
 #endif
 
-PRINT_SETUP()
-
 int nccmp_cmp_att(int ncid1, int ncid2, int varid1, int varid2,
            const char* name, const char* varname, nccmp_opt_t* opts)
 {
@@ -64,32 +62,33 @@ int nccmp_cmp_att(int ncid1, int ncid2, int varid1, int varid2,
             nccmp_pretty_print_att(ncid2, varname, varid2, name, typestr2, MAX_STRLEN);
             PRINT_DIFF(opts->color, opts->debug, " : LENGTHS : ATTRIBUTE : %s : VARIABLE : %s : %lu <> %lu : VALUES : ", name, varname, (unsigned long)lenp1, (unsigned long)lenp2);
         }
+
+        char isEosDiff = 0;
         switch(type1) {
-        case NC_CHAR:
-            if (!opts->quiet) {
-                /* Quote strings. */
-                fprintf(stderr, "\"%s\" : \"%s\"\n", typestr1, typestr2);
-            }
-            if (strcmp(typestr1,typestr2) == 0) {
+            case NC_CHAR:
                 /* Same text, but invisible trailing nulls because lengths differ. */
-                if (opts->warn[NCCMP_W_EOS] || opts->warn[NCCMP_W_ALL]) {
-                    /* Pass */
-                } else {
-                    status = EXIT_DIFFER;
-                    return status;
+                if (strcmp(typestr1,typestr2) == 0) {
+                  isEosDiff = 1;
                 }
-            }
-            break;
-        default:
-            if (!opts->quiet) {
-                /* Unquoted. */
-                fprintf(stderr, "%s : %s\n", typestr1, typestr2);
-            }
-            if (!opts->warn[NCCMP_W_ALL]) {
-                status = EXIT_DIFFER;
-                return status;
-            }
-            break;
+
+                if (!opts->quiet) {
+                    /* Quote strings. */
+                    fprintf(stderr, "\"%s\" : \"%s\"\n", typestr1, typestr2);
+                }
+                break;
+            default:
+                if (!opts->quiet) {
+                    /* Unquoted. */
+                    fprintf(stderr, "%s : %s\n", typestr1, typestr2);
+                }
+                break;
+        }
+
+        if ( isEosDiff &&
+             opts->warn[NCCMP_W_EOS] ) {
+            /* Pass */
+        } else if ( !opts->warn[NCCMP_W_ALL] ) {
+            return EXIT_DIFFER;
         }
     }
     if (nccmp_cmp_att_val(ncid1, ncid2, varid1, varid2, name, lenp1, lenp2, type1) != EXIT_SUCCESS) {
@@ -110,8 +109,7 @@ int nccmp_cmp_att(int ncid1, int ncid2, int varid1, int varid2,
             }
         }
         if (!opts->warn[NCCMP_W_ALL]) {
-            status = EXIT_DIFFER;
-            return status;
+            return EXIT_DIFFER;
         }
     }
     return EXIT_SUCCESS;
@@ -1442,11 +1440,11 @@ int nccmp_dump_info_vars(int ncid)
         HANDLE_NC_ERROR(status);
 
         if (is_nc4) {
-            status = nc_inq_var_szip(ncid, i, & options_mask, & pixels_per_block);
-            HANDLE_NC_ERROR(status);
-        } else {
             options_mask = 0;
             pixels_per_block = 0;
+            status = nc_inq_var_szip(ncid, i, & options_mask, & pixels_per_block);
+            HANDLE_NC_SZIP_ERROR(status);
+            status = NC_NOERR;
         }
 
         nccmp_size_t_array_to_str(chunksizes, ndims, chunksizes_str, ",");
@@ -1521,7 +1519,7 @@ void nccmp_get_rec_names(nccmp_opt_t *opts, int ncid1, int ncid2,
         status = nc_inq_dimname(ncid2, recid2, recname2);
         HANDLE_NC_ERROR(status);
     } else {
-        strcpy(recname1, "");
+        strcpy(recname2, "");
     }
 }
 
@@ -2132,10 +2130,14 @@ int nccmp_var_encoding(nccmp_state_t *state, nccmp_strlist_t* varlist, int ncid1
             }
         }
 
+        szip_mask1 = 0;
+        szip_mask2 = 0;
+        szip_pixels_per_block1 = 0;
+        szip_pixels_per_block2 = 0;
         status = nc_inq_var_szip(ncid1, varid1, & szip_mask1, & szip_pixels_per_block1);
-        HANDLE_NC_ERROR(status);
+        HANDLE_NC_SZIP_ERROR(status);
         status = nc_inq_var_szip(ncid2, varid2, & szip_mask2, & szip_pixels_per_block2);
-        HANDLE_NC_ERROR(status);
+        HANDLE_NC_SZIP_ERROR(status);
 
         if (szip_mask1 != szip_mask2) {
             if (!state->opts.quiet) {
